/*
 * File:   Flash_Functions.c
 * Author: rmcnamee
 *
 * Created on December 8, 2017, 11:09 AM
 */

#include "Flash_Functions.h"
#include "mcc_generated_files/spi1.h"
#include "IO_Functions.h"

// Per SST25VF080B datasheet, this byte signals if the memory is busy with an operation
#define FLASH_BUSY_BIT 0x01

uint24_t flashFramelessImageStartAddress;
uint24_t flashFramelessAttributeStartAddress;
uint24_t flashStandardImageStartAddress;
uint24_t flashStandardAttributeStartAddress;
uint24_t flashDisplayImageStartAddress;
uint24_t flashDisplayAttributeStartAddress;
uint24_t flashRockerImageStartAddress;
uint24_t flashRockerAttributeStartAddress;
uint24_t flashDevKitAddress;

void AssignMemoryLocations(void);
void SendFlashAddress(uint24_t flashAddress);
void Erase4kFlashPage(uint24_t flashAddress);
void Erase32kFlashPage(uint24_t flashAddress);
void Erase64kFlashPage(uint24_t flashAddress);

// Provides the setup needed for correct flash memory operation
void SetupFlash(void)
{
    DisableFlashWriteProtect();
    DisableFlashChipSelect();
    
    AssignMemoryLocations();
    
    return;
}

// Assigns default memory locations for each switch type. Using #DEFINE cut off some of the addresses, so we
// define them here
void AssignMemoryLocations(void)
{
    // For this demo kit, we are going to allow 511 images for each switch type, and there is
    // one attribute per image. The following addresses were calculated based on those numbers    
    flashFramelessImageStartAddress = 0x000000;     // 96x64(x2 bytes/colors) = 12288 bytes, x 512 images = 0x600000 bytes
    flashFramelessAttributeStartAddress = 0x600000; // 16 bytes x512 attributes = 8192 bytes
    flashStandardImageStartAddress = 0x602000;      // 64x48 (x2 bytes/colors) = 6144 bytes, x 512 images = 0x300000 bytes
    //flashStandardAttributeStartAddress = 0x902000;  // 16 bytes x512 attributes = 8192 bytes
    flashStandardAttributeStartAddress = flashFramelessAttributeStartAddress; // For ease of use, most switches will use the same attribute space
    flashDisplayImageStartAddress = 0x904000;       // 52x36 (x2 bytes/colors) = 3744 (we byte align to 3840), x 512 images = 0x1E0000 bytes
    //flashDisplayAttributeStartAddress = 0xAE4000;   // 16 bytes x512 attributes = 8192 bytes
    flashDisplayAttributeStartAddress = flashFramelessAttributeStartAddress; // For ease of use, most switches will use the same attribute space
    flashRockerImageStartAddress = 0xAE6000;        // 1024 bytes (actually 768 but we byte align) x 512 images = 0x080000 bytes
    flashRockerAttributeStartAddress = 0xB66000;    // 16 bytes x512 attributes = 8192 bytes         
    flashDevKitAddress = 0xB68000;                  // 1 byte
}

// Reads the status register from the memory, and returns the status byte
uint8_t  ReadStatusRegister(void)
{
    EnableFlashChipSelect();
    
    uint8_t dummyResponse;
    uint8_t statusByte;
    
    flashCommandsEnum command = FLASH_READ_STATUS_REGISTER;
    
    dummyResponse = SPI1_Exchange8bit(command);        
    
    statusByte = SPI1_Exchange8bit(0xFF);    
        
    DisableFlashChipSelect(); // Per datasheet, drive chip select high after evey command/response           
    
    return statusByte;
}

// Reads the status register of the memory, and checks if the memory is busy
// with an operation. Returns true if the memory is busy, false otherwise
bool CheckIfFlashBusy(void)
{
    uint8_t status = ReadStatusRegister();
    uint8_t busyBit = (uint8_t) (status & FLASH_BUSY_BIT);
    if (busyBit != 0)
    {
        return true;
    }
    else
    {
        return false;
    }
}

// BLOCKING. Continuously polls the memory until it is not busy
void WaitUntilFlashNotBusy(void)
{
    while (CheckIfFlashBusy() == true);
    return;
}

// Sends the write enable command to the memory
void SendWriteEnableCommand(void)
{
    EnableFlashChipSelect();
    flashCommandsEnum command = FLASH_WRITE_ENABLE;
    SPI1_Exchange8bit(command);     
    DisableFlashChipSelect(); // Per datasheet, drive chip select high after evey command/response 
}

// Sends the write disable command to the memory
void SendWriteDisableCommand(void)
{
    EnableFlashChipSelect();
    flashCommandsEnum command = FLASH_WRITE_DISABLE;
    SPI1_Exchange8bit(command);     
    DisableFlashChipSelect(); // Per datasheet, drive chip select high after evey command/response 
}

// DOES NOT CHIP ENABLE. Sends the specified flash address to the memory
void SendFlashAddress(uint24_t flashAddress)
{
    unsigned char highAddressByte = (flashAddress >> 16) & 0xFF;
    unsigned char midAddressByte = (flashAddress >> 8) & 0xFF;
    unsigned char lowAddressByte = (flashAddress >> 0) & 0xFF;
    
    SPI1_Exchange8bit(highAddressByte);
    SPI1_Exchange8bit(midAddressByte);
    SPI1_Exchange8bit(lowAddressByte);  
}

// Writes the data in the provided buffer to the flash address
void WriteDataToFlash(uint24_t flashAddress, unsigned char* dataBuffer, unsigned int numberOfBytesToWrite)
{
    WaitUntilFlashNotBusy();    
    
    SendWriteEnableCommand();
    DisableFlashWriteProtect();        
    EnableFlashChipSelect();    
    
    SPI1_Exchange8bit(FLASH_PAGE_PROGRAM);
    SendFlashAddress(flashAddress);     
    
    for (unsigned int i = 0; i < numberOfBytesToWrite; i++)
    {
        SPI1_Exchange8bit(dataBuffer[i]);
    }
        
    DisableFlashChipSelect();            
    WaitUntilFlashNotBusy();    
}

// Erases a 4k page in the memory
void Erase4kFlashPage(uint24_t flashAddress)
{
    WaitUntilFlashNotBusy();
    
    SendWriteEnableCommand();
    EnableFlashChipSelect();    
    
    flashCommandsEnum command = FLASH_ERASE_4K_PAGE;
    SPI1_Exchange8bit(command);  
    
    SendFlashAddress(flashAddress);  
    
    DisableFlashChipSelect();                
}

// Erases a 32k page in the memory
void Erase32kFlashPage(uint24_t flashAddress)
{
    WaitUntilFlashNotBusy();
    
    SendWriteEnableCommand();
    EnableFlashChipSelect();    
    
    flashCommandsEnum command = FLASH_ERASE_32K_PAGE;
    SPI1_Exchange8bit(command);  
    
    SendFlashAddress(flashAddress);  
    
    DisableFlashChipSelect();                
}

// Erases a 64k page in the memory
void Erase64kFlashPage(uint24_t flashAddress)
{
    WaitUntilFlashNotBusy();
    
    SendWriteEnableCommand();
    EnableFlashChipSelect();    
    
    flashCommandsEnum command = FLASH_ERASE_64K_PAGE;
    SPI1_Exchange8bit(command);  
    
    SendFlashAddress(flashAddress);  
    
    DisableFlashChipSelect();                
}

// Erases the entire flash memory chip
void EraseAllFlashMemory(void)
{
    WaitUntilFlashNotBusy();
    
    SendWriteEnableCommand();
    EnableFlashChipSelect();    
    
    flashCommandsEnum command = FLASH_ERASE_FULL_CHIP;
    SPI1_Exchange8bit(command);              
    
    DisableFlashChipSelect();    
    WaitUntilFlashNotBusy();
}

// Reads from flash memory at the specified address, the puts the data in the array given
void ReadFlashMemory(uint24_t flashAddress, unsigned char* dataBuffer, unsigned int numberOfBytesToRead)
{
    WaitUntilFlashNotBusy();
    
    EnableFlashChipSelect();    
    
    flashCommandsEnum command = FLASH_READ;
    SPI1_Exchange8bit(command);   
    
    SendFlashAddress(flashAddress);  
    
    for (unsigned int i = 0; i < numberOfBytesToRead; i++)
    {
        dataBuffer[i] = SPI1_Exchange8bit(0xFF);
    }
    
    DisableFlashChipSelect();
    
    return;    
}

// This flash memory doesn't need repeated commands to keep reading, so send the read start address
// and keep the chip select low for repeated reads
void StartReadFlashMemory(uint24_t flashAddress)
{
    WaitUntilFlashNotBusy();
    
    EnableFlashChipSelect();    
    
    flashCommandsEnum command = FLASH_READ;
    SPI1_Exchange8bit(command);   
    
    SendFlashAddress(flashAddress);
}

// StartReadFlashMemory(uint24_t flashAddress) MUST HAVE BEEN CALLED FIRST
// Reads the next sequential byte in flash memory
uint8_t ContinueReadFlashMemory(void)
{
    return SPI1_Exchange8bit(0xFF);
}

// Ends the sequential read by setting the chip select high
void FinishReadFlashMemory(void)
{
    DisableFlashChipSelect();
}

// Erases the section of memory containing all of the flash attributes
void EraseAllFlashAttributes(switchTypeEnum type)
{
//    uint24_t attributeStartAddress = GetAttributeStartAddress(type);
//    for (uint24_t address = attributeStartAddress; address < (address + 0x100); address += 0x40)
//    {
//        Erase64kFlashPage(address);   
//    }    
}

// Erases the section of memory containing all of the images
void EraseAllFlashImages(switchTypeEnum type)
{
//    uint24_t imageStartAddress = GetImageStartAddress(type);
//    for (uint24_t address = imageStartAddress; address < 0x7FFFF; address += 0xFFFF)
//    {
//        Erase64kFlashPage(address);   
//    }    
}

// The start address depends on the dev kit being used, so return the correct
// address for the dev kit being used (changed in GetCurrentDevKit() in main.c)
uint24_t GetSwitch1ImageStartAddress(void)
{
    devKitTypeEnum type = GetCurrentDevKit();
    
    switch(type)
    {
        default:
        case DEV_KIT_TWO_FRAMELESS:
            return flashFramelessImageStartAddress;
        case DEV_KIT_ONE_FRAMELESS_ONE_STANDARD:
            return flashFramelessImageStartAddress;
        case DEV_KIT_ONE_STANDARD_ONE_DISPLAY:
            return flashDisplayImageStartAddress;
        case DEV_KIT_ROCKER:
            return flashRockerImageStartAddress;
    }
}

// The start address depends on the dev kit being used, so return the correct
// address for the dev kit being used (changed in GetCurrentDevKit() in main.c
uint24_t GetSwitch1AttributeStartAddress(void)
{
    devKitTypeEnum type = GetCurrentDevKit();
    
    switch(type)
    {
        default:
        case DEV_KIT_TWO_FRAMELESS: // Switch 1 is frameless in this kit
            return flashFramelessAttributeStartAddress;
        case DEV_KIT_ONE_FRAMELESS_ONE_STANDARD: // Switch 1 is frameless in this kit
            return flashFramelessAttributeStartAddress;
        case DEV_KIT_ONE_STANDARD_ONE_DISPLAY: // Switch 1 is a display in this kit
            return flashDisplayAttributeStartAddress;
        case DEV_KIT_ROCKER: // Switch 1 is a rocker in this kit
            return flashRockerAttributeStartAddress;
    }
}

// The start address depends on the dev kit being used, so return the correct
// address for the dev kit being used (changed in GetCurrentDevKit() in main.c
uint24_t GetSwitch2ImageStartAddress(void)
{
    devKitTypeEnum type = GetCurrentDevKit();
    
    switch(type)
    {
        default:
        case DEV_KIT_TWO_FRAMELESS: // Switch 2 is frameless in this kit
            return flashFramelessImageStartAddress;
        case DEV_KIT_ONE_FRAMELESS_ONE_STANDARD: // Switch 2 is standard in this kit
            return flashStandardImageStartAddress;
        case DEV_KIT_ONE_STANDARD_ONE_DISPLAY: // Switch 2 is standard in this kit
            return flashStandardImageStartAddress;
        case DEV_KIT_ROCKER: // There is no switch 2 in this kit
            return flashRockerImageStartAddress;
    }
}

// The start address depends on the dev kit being used, so return the correct
// address for the dev kit being used (changed in GetCurrentDevKit() in main.c
uint24_t GetSwitch2AttributeStartAddress(void)
{
    devKitTypeEnum type = GetCurrentDevKit();
    
    switch(type)
    {
        default:
        case DEV_KIT_TWO_FRAMELESS:
            return flashFramelessAttributeStartAddress;
        case DEV_KIT_ONE_FRAMELESS_ONE_STANDARD:
            return flashStandardAttributeStartAddress;
        case DEV_KIT_ONE_STANDARD_ONE_DISPLAY:
            return flashStandardAttributeStartAddress;
        case DEV_KIT_ROCKER:
            return flashRockerAttributeStartAddress;
    }
}

// The start address depends on the dev kit being used, so return the correct
// address for the dev kit being used (changed in GetCurrentDevKit() in main.c
uint24_t GetRockerImageStartAddress(void)
{
    return flashRockerImageStartAddress;
}

// The start address depends on the dev kit being used, so return the correct
// address for the dev kit being used (changed in GetCurrentDevKit() in main.c
uint24_t GetRockerAttributeStartAddress(void)
{
    return flashRockerAttributeStartAddress;
}

// Returns the memory start address for the specified switch type
uint24_t GetImageStartAddress(switchTypeEnum type)
{
    switch(type)
    {
        default:
        case SWITCH_TYPE_96x64_OLED:
            return flashFramelessImageStartAddress;
        case SWITCH_TYPE_64x48_OLED:
            return flashStandardImageStartAddress;
        case SWITCH_TYPE_52x36_OLED:
            return flashDisplayImageStartAddress;
        case SWITCH_TYPE_OLED_ROCKER:
            return flashRockerImageStartAddress;
    }
}

// Returns the memory start address for the specified switch type
uint24_t GetAttributeStartAddress(switchTypeEnum type)
{
    switch(type)
    {
        default:
        case SWITCH_TYPE_96x64_OLED:
            return flashFramelessAttributeStartAddress;
        case SWITCH_TYPE_64x48_OLED:
            return flashStandardAttributeStartAddress;
        case SWITCH_TYPE_52x36_OLED:
            return flashDisplayAttributeStartAddress;
        case SWITCH_TYPE_OLED_ROCKER:
            return flashRockerAttributeStartAddress;
    }
}

