/*
 * File:   Setup_Program.c
 * Author: rmcnamee
 *
 * Created on December 13, 2017, 10:29 AM
 */

#include "Switch_Functions.h"
#include "Flash_Functions.h"
#include "IO_Functions.h"
#include "mcc_generated_files/spi2.h"
#include "Timer_Functions.h"
#include "Communication_Handler.h"
#include "FontDefinitions.h"

#define BACKLIGHT_COLOR_TIME_ON 1000 // Set a time of 1 second between changing backlight colors (For demonstration)

bool test;

void ConfigureSwitchInialSettings(void);
void ReadImageAttribute(uint24_t flashAddress, imageAttributeStruct* attributeStruct);
void LoadStartingImageAndAttribute(void);
void LoadSwitch1Image(uint24_t attributeAddress);
void LoadSwitch2Image(uint24_t attributeAddress);
void LoadRockerImage(uint24_t imageAddress);
void SendImageToSwitches(uint24_t flashAddress, unsigned int imageSize);
void SendImageToRocker(uint24_t flashAddress, unsigned int imageSize);
void DoSwitch1PressedBehavior(void);
void DoSwitch2PressedBehavior(void);
void DoRockerPressedBehavior(rockerPressEnum rockerPress);
void CheckIfSwitch1LoopTimerNeedsStarting(void);
void CheckIfSwitch2LoopTimerNeedsStarting(void);
void CheckIfRockerLoopTimerNeedsStarting(void);
void ConfigureFramelessOledSwitch(void);
void ConfigureStandardOledSwitch(void);
void ConfigureOledDisplay(void);
void ConfigureOledRocker(void);
void SendSwitch1ImageChangedNotification(uint24_t imageAddress);
void SendSwitch2ImageChangedNotification(uint24_t imageAddress);
void SendRockerPressedResponse(rockerPressEnum rockerPress);
void SendRockerReleasedResponse(rockerPressEnum rockerPress);

imageAttributeStruct switch1ImageAttributes; // A struct to hold the attributes for switch 1
imageAttributeStruct switch2ImageAttributes; // A struct to hold the attributes for switch 2
imageAttributeStruct rockerImageAttributes; // A struct to hold the attributes for switch 2

bool switch1PressedPreviously; // For debouncing, don't want to trigger on user holding down
bool switch2PressedPreviously; // For debouncing, don't want to trigger on user holding down
bool rockerPressedPreviously; // For debouncing, don't want to trigger on user holding down

bool switch1AttributeTableInvalid;
bool switch2AttributeTableInvalid;
bool rockerAttributeTableInvalid;

rockerPressEnum rockerPreviouslyPressedSwitch;

// Provides the setup needed for correct switch operation
void SetupSwitches(void)
{    
    DisableSwitchCommandSending();
    DisableSwitch1ChipSelect();
    DisableSwitch2ChipSelect();  
    DisableRockerChipSelect();
    DisableSwitchReset(); // Take the switches out of reset before doing any configuration
            
    ConfigureSwitchInialSettings();
    
    EnablePowerToSwitches(); // Needs to be after all configuration sent
    LoadStartingImageAndAttribute();    
    
    StartTimer(TIMER_SWITCH_SCAN, 25);
}

// Sends commands to both switches to configure them initially
void ConfigureSwitchInialSettings(void)
{        
    devKitTypeEnum devKitType = GetCurrentDevKit();
    
    switch (devKitType)
    {
        default:
        case DEV_KIT_TWO_FRAMELESS:
            EnableSwitch1ChipSelect();
            EnableSwitch2ChipSelect();
            
            ConfigureFramelessOledSwitch(); // Both switches are frameless
            
            DisableSwitch1ChipSelect();
            DisableSwitch2ChipSelect(); 
            break;;
        case DEV_KIT_ONE_FRAMELESS_ONE_STANDARD:            
            EnableSwitch1ChipSelect();
            ConfigureFramelessOledSwitch(); // Switch 1 is frameless
            DisableSwitch1ChipSelect();
            
            EnableSwitch2ChipSelect();
            ConfigureStandardOledSwitch(); // Switch 2 is standard                     
            DisableSwitch2ChipSelect(); 
            break;
        case DEV_KIT_ONE_STANDARD_ONE_DISPLAY:
            EnableSwitch1ChipSelect();
            ConfigureOledDisplay(); // Switch 1 is display
            DisableSwitch1ChipSelect();
            
            EnableSwitch2ChipSelect();
            ConfigureStandardOledSwitch(); // Switch 2 is standard                     
            DisableSwitch2ChipSelect(); 
            break;
        case DEV_KIT_ROCKER:            
            ConfigureOledRocker();            
            break;
    }                
}

// <editor-fold defaultstate="collapsed" desc="Switch Configuration Functions">
// Sends a command and its parameters to both switches
void SendSwitchConfigurationCommand(unsigned char command, unsigned char* parameters, unsigned char numberOfParameters)
{
    EnableSwitchCommandSending();
    
    SPI2_Exchange8bit(command);
    
    for (unsigned char i = 0; i < numberOfParameters; i++)
    {
        SPI2_Exchange8bit(parameters[i]);
    }
    
    DisableSwitchCommandSending();
}

// Sends a command and its parameters to the rocker
void SendRockerConfigurationCommand(unsigned char command, unsigned char* parameters, unsigned char numberOfParameters)
{
    EnableRockerChipSelect();
    EnableSwitchCommandSending();    
    
    SPI2_Exchange8bit(command);
    DisableSwitchCommandSending();
    
    for (unsigned char i = 0; i < numberOfParameters; i++)
    {
        SPI2_Exchange8bit(parameters[i]);
    }
    
    DisableRockerChipSelect();
}

void ConfigureFramelessOledSwitch(void)
{
    unsigned char parameters[5] = {0};
    
    parameters[0] = FRAMELESS_CONTRAST_A_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_A, parameters, 1);
            
    parameters[0] = FRAMELESS_CONTRAST_B_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_B, parameters, 1);   
            
    parameters[0] = FRAMELESS_CONTRAST_C_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_C, parameters, 1);  
            
    parameters[0] = FRAMELESS_MASTER_CURRENT_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MASTER_CURRENT_CONTROL, parameters, 1);  
    
    parameters[0] = FRAMELESS_COLOR_A_PRECHARGE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_COLOR_A_PRECHARGE_SPEED, parameters, 1);  
    
    parameters[0] = FRAMELESS_COLOR_B_PRECHARGE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_COLOR_B_PRECHARGE_SPEED, parameters, 1);  
   
    parameters[0] = FRAMELESS_COLOR_C_PRECHARGE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_COLOR_C_PRECHARGE_SPEED, parameters, 1);  
            
    parameters[0] = FRAMELESS_COLOR_DEPTH_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_COLOR_DEPTH, parameters, 1);
    
    parameters[0] = FRAMELESS_DISPLAY_START_LINE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_START_LINE, parameters, 1);
    
    parameters[0] = FRAMELESS_DISPLAY_OFFSET_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_OFFSET, parameters, 1);
    
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_NORMAL, parameters, 0);
    
    parameters[0] = FRAMELESS_MULTIPLEX_RATIO_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MULTIPLEX_RATIO, parameters, 1);
    
    parameters[0] = FRAMELESS_DIM_MODE_DEFAULT_1;
    parameters[1] = FRAMELESS_DIM_MODE_DEFAULT_2;
    parameters[2] = FRAMELESS_DIM_MODE_DEFAULT_3;
    parameters[3] = FRAMELESS_DIM_MODE_DEFAULT_4;
    parameters[4] = FRAMELESS_DIM_MODE_DEFAULT_5;
    SendSwitchConfigurationCommand(SWITCH_COLOR_DIM_MODE, parameters, 5);
   
    parameters[0] = FRAMELESS_MASTER_CONFIGURATION_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MASTER_CONFIGURATION, parameters, 1);
    
    parameters[0] = FRAMELESS_POWER_SAVE_MODE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_POWER_SAVE_MODE, parameters, 1);
    
    parameters[0] = FRAMELESS_PHASE_PERIOD_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_PHASE_1_2_PERIOD_ADJUST, parameters, 1);
    
    parameters[0] = FRAMELESS_DISPLAY_CLOCK_DIVIDER_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_CLOCK_DIVIDER, parameters, 1);
    
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_ENABLE_LINEAR_GRAYSCALE, parameters, 0);
    
    parameters[0] = FRAMELESS_PRECHARGE_LEVEL_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_PRECHARGE_LEVEL, parameters, 1);
    
    parameters[0] = FRAMELESS_VCOMH_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_SET_VCOMH, parameters, 1);
  
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_ON, parameters, 0);
   
    parameters[0] = FRAMELESS_WINDOW_COLUMN_DEFAULT_1;
    parameters[1] = FRAMELESS_WINDOW_COLUMN_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_COLUMN, parameters, 2);
   
    parameters[0] = FRAMELESS_WINDOW_ROW_DEFAULT_1;
    parameters[1] = FRAMELESS_WINDOW_ROW_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_ROW, parameters, 2);    
}

void ConfigureStandardOledSwitch(void)
{
    unsigned char parameters[5] = {0};
    
    parameters[0] = STANDARD_CONTRAST_A_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_A, parameters, 1);
    
    parameters[0] = STANDARD_CONTRAST_B_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_B, parameters, 1);   
    
    parameters[0] = STANDARD_CONTRAST_C_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_C, parameters, 1); 
    
    parameters[0] = STANDARD_MASTER_CURRENT_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MASTER_CURRENT_CONTROL, parameters, 1);  
    
    parameters[0] = STANDARD_COLOR_DEPTH_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_COLOR_DEPTH, parameters, 1);
    
    parameters[0] = STANDARD_DISPLAY_START_LINE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_START_LINE, parameters, 1);
    
    parameters[0] = STANDARD_DISPLAY_OFFSET_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_OFFSET, parameters, 1);
    
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_NORMAL, parameters, 0);
    
    parameters[0] = STANDARD_MULTIPLEX_RATIO_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MULTIPLEX_RATIO, parameters, 1);
    
    parameters[0] = STANDARD_DIM_MODE_DEFAULT_1;
    parameters[1] = STANDARD_DIM_MODE_DEFAULT_2;
    parameters[2] = STANDARD_DIM_MODE_DEFAULT_3;
    parameters[3] = STANDARD_DIM_MODE_DEFAULT_4;
    parameters[4] = STANDARD_DIM_MODE_DEFAULT_5;
    SendSwitchConfigurationCommand(SWITCH_COLOR_DIM_MODE, parameters, 5);
    
    parameters[0] = STANDARD_MASTER_CONFIGURATION_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MASTER_CONFIGURATION, parameters, 1);
    
    parameters[0] = STANDARD_POWER_SAVE_MODE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_POWER_SAVE_MODE, parameters, 1);
    
    parameters[0] = STANDARD_PHASE_PERIOD_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_PHASE_1_2_PERIOD_ADJUST, parameters, 1);
    
    parameters[0] = STANDARD_DISPLAY_CLOCK_DIVIDER_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_CLOCK_DIVIDER, parameters, 1);
    
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_ENABLE_LINEAR_GRAYSCALE, parameters, 0);
  
    parameters[0] = STANDARD_PRECHARGE_LEVEL_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_PRECHARGE_LEVEL, parameters, 1);
    
    parameters[0] = STANDARD_VCOMH_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_SET_VCOMH, parameters, 1);
    
    
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_ON, parameters, 0);
    
    parameters[0] = STANDARD_WINDOW_COLUMN_DEFAULT_1;
    parameters[1] = STANDARD_WINDOW_COLUMN_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_COLUMN, parameters, 2);
   
    parameters[0] = STANDARD_WINDOW_ROW_DEFAULT_1;
    parameters[1] = STANDARD_WINDOW_ROW_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_ROW, parameters, 2);           
}

void ConfigureOledDisplay(void)
{
    unsigned char parameters[5] = {0};
    
    parameters[0] = DISPLAY_CONTRAST_A_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_A, parameters, 1);
    
    parameters[0] = DISPLAY_CONTRAST_B_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_B, parameters, 1);   
    
    parameters[0] = DISPLAY_CONTRAST_C_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_CONTRAST_COLOR_C, parameters, 1);  
    
    parameters[0] = DISPLAY_MASTER_CURRENT_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MASTER_CURRENT_CONTROL, parameters, 1);  
    
    parameters[0] = DISPLAY_COLOR_DEPTH_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_COLOR_DEPTH, parameters, 1);  
    
    parameters[0] = DISPLAY_DISPLAY_START_LINE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_START_LINE, parameters, 1);
    
    parameters[0] = DISPLAY_DISPLAY_OFFSET_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_OFFSET, parameters, 1);
    
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_NORMAL, parameters, 0);
    
    parameters[0] = DISPLAY_MULTIPLEX_RATIO_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MULTIPLEX_RATIO, parameters, 1);
    
    parameters[0] = DISPLAY_DIM_MODE_DEFAULT_1;
    parameters[1] = DISPLAY_DIM_MODE_DEFAULT_2;
    parameters[2] = DISPLAY_DIM_MODE_DEFAULT_3;
    parameters[3] = DISPLAY_DIM_MODE_DEFAULT_4;
    parameters[4] = DISPLAY_DIM_MODE_DEFAULT_5;
    SendSwitchConfigurationCommand(SWITCH_COLOR_DIM_MODE, parameters, 5);
    
    parameters[0] = DISPLAY_MASTER_CONFIGURATION_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_MASTER_CONFIGURATION, parameters, 1);
    
    parameters[0] = DISPLAY_POWER_SAVE_MODE_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_POWER_SAVE_MODE, parameters, 1);
    
    parameters[0] = DISPLAY_PHASE_PERIOD_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_PHASE_1_2_PERIOD_ADJUST, parameters, 1);
    
    parameters[0] = DISPLAY_DISPLAY_CLOCK_DIVIDER_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_CLOCK_DIVIDER, parameters, 1);
        
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_ENABLE_LINEAR_GRAYSCALE, parameters, 0);
    
    parameters[0] = DISPLAY_PRECHARGE_LEVEL_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_PRECHARGE_LEVEL, parameters, 1);
    
    parameters[0] = DISPLAY_VCOMH_DEFAULT;
    SendSwitchConfigurationCommand(SWITCH_SET_VCOMH, parameters, 1);
        
    parameters[0] = 0x00;
    SendSwitchConfigurationCommand(SWITCH_DISPLAY_ON, parameters, 0);
    
    parameters[0] = DISPLAY_WINDOW_COLUMN_DEFAULT_1;
    parameters[1] = DISPLAY_WINDOW_COLUMN_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_COLUMN, parameters, 2);
    
    parameters[0] = DISPLAY_WINDOW_ROW_DEFAULT_1;
    parameters[1] = DISPLAY_WINDOW_ROW_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_ROW, parameters, 2);        
}

void ConfigureOledRocker(void)
{
    unsigned char parameters[2] = {0, 0};
    
    SendRockerConfigurationCommand(ROCKER_RESET,parameters,1);
    
    parameters[0] = ROCKER_DISPLAY_OFF_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_DISPLAY_ON_OFF,parameters,1);   
    
    parameters[0] = ROCKER_READ_WRITE_WETTING_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_READ_WRITE_WETTING,parameters,1);   
    
    parameters[0] = ROCKER_DISPLAY_DIRECTION_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_SET_DISPLAY_DIRECTION,parameters,1); 
 
    parameters[0] = ROCKER_RESERVED1_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED1,parameters,1); 

    parameters[0] = ROCKER_RESERVED2_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED2,parameters,1); 
     
    parameters[0] = ROCKER_RESERVED3_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED3,parameters,1); 
 
    parameters[0] = ROCKER_DISPLAY_STANDBY_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_DISPLAY_STANDBY,parameters,1);         
    
    parameters[0] = ROCKER_RESERVED4_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED4,parameters,1); 

    parameters[0] = ROCKER_RESERVED5_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED5,parameters,1); 
    
    parameters[0] = ROCKER_RESERVED6_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED6,parameters,1); 

    parameters[0] = ROCKER_RESERVED7_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED7,parameters,1); 
 
    parameters[0] = ROCKER_RESERVED8_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED8,parameters,1); 

    parameters[0] = ROCKER_WRITE_DIRECTION_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_WRITE_DIRECTION,parameters,1); 
    
    parameters[0] = ROCKER_COLUMN_OUTPUT_RANGE_DEFAULT1;
    parameters[1] = ROCKER_COLUMN_OUTPUT_RANGE_DEFAULT2;
    SendRockerConfigurationCommand(ROCKER_COLUMN_OUPUT_RANGE,parameters,2);     
    
    parameters[0] = ROCKER_ROW_OUTPUT_RANGE_DEFAULT1;
    parameters[1] = ROCKER_ROW_OUTPUT_RANGE_DEFAULT2;
    SendRockerConfigurationCommand(ROCKER_ROW_OUTPUT_RANGE,parameters,2);     
     
    parameters[0] = ROCKER_X_AXIS_START_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_X_AXIS_START,parameters,1); 
    
    parameters[0] = ROCKER_X_AXIS_END_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_X_AXIS_END,parameters,1);   
    
    parameters[0] = ROCKER_Y_AXIS_START_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_Y_AXIS_START,parameters,1); 
        
    parameters[0] = ROCKER_Y_AXIS_END_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_Y_AXIS_END,parameters,1); 
    
    parameters[0] = ROCKER_X_AXIS_ADDRESS_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_X_AXIS_ADDRESS,parameters,1); 
    
    parameters[0] = ROCKER_Y_AXIS_ADDRESS_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_Y_AXIS_ADDRESS,parameters,1); 
    
    parameters[0] = ROCKER_RESERVED9_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED9,parameters,1); 
        
    parameters[0] = ROCKER_SCREEN_SAVER_SETTING1_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_SCREEN_SAVER_SETTING1,parameters,1); 
    
    parameters[0] = ROCKER_SCREEN_SAVER_SETTING2_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_SCREEN_SAVER_SETTING2,parameters,1); 
    
    parameters[0] = ROCKER_SCREEN_SAVER_MODE_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_SCREEN_SAVER_MODE,parameters,1); 
    
    parameters[0] = ROCKER_SCREEN_SAVER_STOP_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_SCREEN_SAVER_START_STOP,parameters,1); 
    
    parameters[0] = ROCKER_CLOCK_DIVISION_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_CLOCK_DIVISION,parameters,1); 
    
    parameters[0] = ROCKER_STBY_PIN_SETTING_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_STBY_PIN_SETTING,parameters,1); 
    
    parameters[0] = ROCKER_DACA_SETTING_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_DACA_SETTING,parameters,1);     
    
    parameters[0] = ROCKER_DACB_SETTING_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_DACB_SETTING,parameters,1);     
    
    parameters[0] = ROCKER_DACC_SETTING_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_DACC_SETTING,parameters,1);     
    
    parameters[0] = ROCKER_DACD_SETTING_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_DACD_SETTING,parameters,1);     
    
    parameters[0] = ROCKER_BRIGHTNESS_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_BRIGHTNESS,parameters,1);     
    
    parameters[0] = ROCKER_DIMMER_SETTING_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_DIMMER_SETTING,parameters,1);     
    
    parameters[0] = ROCKER_RESERVED11_DEFAULT;
    SendRockerConfigurationCommand(ROCKER_RESERVED11,parameters,1);     
    
    parameters[0] = ROCKER_DISPLAY_ON_PARAMETER;
    SendRockerConfigurationCommand(ROCKER_DISPLAY_ON_OFF,parameters,1);       
}

// </editor-fold>

// <editor-fold defaultstate="collapsed" desc="Attribute Read/Write Functions">
// Reads the setup attribute (0x00) and configures appropriately
void ReadSetupAttributes(void)
{
    unsigned char attributeData[16];
    uint24_t flashAddress;
    uint24_t attributeSize;
    uint24_t switch1AttributeStartAddress;
    uint24_t switch2AttributeStartAddress;
    uint24_t rockerAttributeStartAddress;
    
    switch1AttributeStartAddress = GetSwitch1AttributeStartAddress();
    switch2AttributeStartAddress = GetSwitch2AttributeStartAddress();
    rockerAttributeStartAddress = GetRockerAttributeStartAddress();
    
    attributeSize = GetCurrentSwitchAttributeSize();
    
    ReadFlashMemory(switch1AttributeStartAddress,attributeData,attributeSize); // Read setup attribute for switch 1 (attribute 0x0000)
    if (attributeData[SETUP_SETUP_ADDRESS_HIGH] == FLASH_ATTRIBUTE_NOT_PROGRAMMED_BYTE) // Load default values if no attribute table has been loaded
    {
        switch1ImageAttributes.currentAddress = 0x0001;        
    }
    else
    {
        switch1ImageAttributes.currentAddress = 0x0001; // We always set the first switch to be image 1 so the user doesn't get lost        
        SetSwitchBrightness(attributeData[SETUP_BRIGHTNESS]);
    }    
    flashAddress = (switch1ImageAttributes.currentAddress * attributeSize) + switch1AttributeStartAddress; // We define the flash attributes to be in a specific part of memory    
    ReadImageAttribute(flashAddress, &switch1ImageAttributes); // Loads the attributes into the struct
    
    
    ReadFlashMemory(switch2AttributeStartAddress,attributeData,GetCurrentSwitchAttributeSize()); // Read setup attribute for switch 2 (attribute 0x0000)    
    if (attributeData[SETUP_SETUP_ADDRESS_HIGH] == FLASH_ATTRIBUTE_NOT_PROGRAMMED_BYTE) // Load default values if no attribute table has been loaded
    {
        switch2ImageAttributes.currentAddress = 0x0002;
    }
    else
    {
        switch2ImageAttributes.currentAddress = (uint24_t)(attributeData[SETUP_SWITCH2_START_ADDRESS_HIGH] << 8) | attributeData[SETUP_SWITCH2_START_ADDRESS_LOW];      
    }          
    flashAddress = (switch2ImageAttributes.currentAddress * attributeSize) + switch2AttributeStartAddress;
    ReadImageAttribute(flashAddress, &switch2ImageAttributes); // Loads the attributes into the struct   
    
    
    ReadFlashMemory(rockerAttributeStartAddress,attributeData,GetCurrentSwitchAttributeSize()); // Read setup attribute for rocker (attribute 0x0000)    
    if (attributeData[SETUP_SETUP_ADDRESS_HIGH] == FLASH_ATTRIBUTE_NOT_PROGRAMMED_BYTE) // Load default values if no attribute table has been loaded
    {
        rockerImageAttributes.currentAddress = 0x0001; 
    }
    else
    {
        rockerImageAttributes.currentAddress = 0x0001; // We always set the first switch to be image 1 so the user doesn't get lost
        SetSwitchBrightness(attributeData[SETUP_BRIGHTNESS]);
    }       
    flashAddress = (rockerImageAttributes.currentAddress * attributeSize) + rockerAttributeStartAddress;
    ReadImageAttribute(flashAddress, &rockerImageAttributes); // Loads the attributes into the struct     
}

// Loads the starting attribute and image
void LoadStartingImageAndAttribute(void)
{        
    ReadSetupAttributes();
    
    if (switch1ImageAttributes.currentAddress > SWITCH_MAX_IMAGES) // This byte will be 0xFF if an attribute table has not been loaded
    {
        switch1AttributeTableInvalid = true;
        
        switch1ImageAttributes.currentAddress = 0x0001;
    }
    
    if (switch2ImageAttributes.currentAddress > SWITCH_MAX_IMAGES) // This byte will be 0xFF if an attribute table has not been loaded
    {
        switch2AttributeTableInvalid = true;
        
        switch2ImageAttributes.currentAddress = 0x0002;
    }
    
    if (rockerImageAttributes.currentAddress > SWITCH_MAX_IMAGES) // This byte will be 0xFF if an attribute table has not been loaded
    {
        rockerAttributeTableInvalid = true;
        
        rockerImageAttributes.currentAddress = 0x0001;
    }
    
    if (GetCurrentDevKit() == DEV_KIT_ROCKER)
    {
        LoadRockerImage(rockerImageAttributes.currentAddress);
        CheckIfRockerLoopTimerNeedsStarting();
    }
    else
    {
        LoadSwitch1Image(switch1ImageAttributes.currentAddress);  // Loads the image on to the switch
        LoadSwitch2Image(switch2ImageAttributes.currentAddress);
    
        CheckIfSwitch1LoopTimerNeedsStarting();
        CheckIfSwitch2LoopTimerNeedsStarting();
    }                  
}

void StoreOledAttributes(uint24_t flashAddress, imageAttributeStruct* attributeStruct)
{
    unsigned char attributeData[16];    
    ReadFlashMemory(flashAddress,attributeData,GetCurrentSwitchAttributeSize());
 
    attributeStruct->currentAddress = (((uint24_t)(attributeData[ATTRIBUTE_CURRENT_ADDRESS_HIGH])) << 8) | attributeData[ATTRIBUTE_CURRENT_ADDRESS_LOW];
    attributeStruct->loopEndAddress = (((uint24_t)(attributeData[ATTRIBUTE_LOOP_END_ADDRESS_HIGH])) << 8) | attributeData[ATTRIBUTE_LOOP_END_ADDRESS_LOW];
    attributeStruct->timer1 = attributeData[ATTRIBUTE_TIMER1];
    attributeStruct->timer2 = attributeData[ATTRIBUTE_TIMER2];
    attributeStruct->switch1PressAddress = (((uint24_t)(attributeData[ATTRIBUTE_SW1_PRESS_ADDRESS_HIGH])) << 8) | attributeData[ATTRIBUTE_SW1_PRESS_ADDRESS_LOW];
    attributeStruct->switch2PressAddress = (((uint24_t)(attributeData[ATTRIBUTE_SW2_PRESS_ADDRESS_HIGH])) << 8) | attributeData[ATTRIBUTE_SW2_PRESS_ADDRESS_LOW];
    attributeStruct->switch1TimerExpiredJumpAddress = (((uint24_t)(attributeData[ATTRIBUTE_SW1_TIMER_EXPIRED_JUMP_ADDRESS_HIGH])) << 8) | attributeData[ATTRIBUTE_SW1_TIMER_EXPIRED_JUMP_ADDRESS_LOW];
    attributeStruct->switch2TimerExpiredJumpAddress = (((uint24_t)(attributeData[ATTRIBUTE_SW2_TIMER_EXPIRED_JUMP_ADDRESS_HIGH])) << 8) | attributeData[ATTRIBUTE_SW2_TIMER_EXPIRED_JUMP_ADDRESS_LOW];
    attributeStruct->reserved1 = attributeData[ATTRIBUTE_RESERVED1];
    attributeStruct->reserved2 = attributeData[ATTRIBUTE_RESERVED2];
    
    return;    
}

void StoreRockerAttributes(uint24_t flashAddress, imageAttributeStruct* attributeStruct)
{
    unsigned char attributeData[16];    
    ReadFlashMemory(flashAddress,attributeData,GetCurrentSwitchAttributeSize());
 
    attributeStruct->currentAddress = (((uint24_t)(attributeData[ROCKER_ATTRIBUTE_CURRENT_ADDRESS_HIGH])) << 8) | attributeData[ROCKER_ATTRIBUTE_CURRENT_ADDRESS_LOW];
    attributeStruct->loopEndAddress = (((uint24_t)(attributeData[ROCKER_ATTRIBUTE_LOOP_END_ADDRESS_HIGH])) << 8) | attributeData[ROCKER_ATTRIBUTE_LOOP_END_ADDRESS_LOW];
    attributeStruct->timer1 = attributeData[ROCKER_ATTRIBUTE_TIMER1];
    attributeStruct->timer2 = attributeData[ROCKER_ATTRIBUTE_TIMER2];
    attributeStruct->rockerTopPressAddress = (((uint24_t)(attributeData[ROCKER_ATTRIBUTE_SW1_PRESS_ADDRESS_HIGH])) << 8) | attributeData[ROCKER_ATTRIBUTE_SW1_PRESS_ADDRESS_LOW];
    attributeStruct->rockerMiddlePressAddress = (((uint24_t)(attributeData[ROCKER_ATTRIBUTE_SW2_PRESS_ADDRESS_HIGH])) << 8) | attributeData[ROCKER_ATTRIBUTE_SW2_PRESS_ADDRESS_LOW];
    attributeStruct->rockerBottomPressAddress = (((uint24_t)(attributeData[ROCKER_ATTRIBUTE_SW3_PRESS_ADDRESS_HIGH])) << 8) | attributeData[ROCKER_ATTRIBUTE_SW3_PRESS_ADDRESS_LOW];
    attributeStruct->switch1TimerExpiredJumpAddress = (((uint24_t)(attributeData[ROCKER_ATTRIBUTE_TIMER_EXPIRED_JUMP_ADDRESS_HIGH])) << 8) | attributeData[ROCKER_ATTRIBUTE_TIMER_EXPIRED_JUMP_ADDRESS_LOW];
    attributeStruct->reserved1 = attributeData[ROCKER_ATTRIBUTE_RESERVED1];
    attributeStruct->reserved2 = attributeData[ROCKER_ATTRIBUTE_RESERVED2];
    
    return;    
}

// Reads the attribute at the given flash address and loads all the values into the given struct
void ReadImageAttribute(uint24_t flashAddress, imageAttributeStruct* attributeStruct)
{
    if (GetCurrentDevKit() == DEV_KIT_ROCKER) // The rocker has a different attribute table setup
    {
        StoreRockerAttributes(flashAddress, attributeStruct);
    }
    else
    {
        StoreOledAttributes(flashAddress, attributeStruct);
    }
}

// </editor-fold>

// <editor-fold defaultstate="collapsed" desc="Image Read/Write Functions">
// Loads the image at the selected address (0001, 0002, etc.) on switch 1
void LoadSwitch1Image(uint24_t imageAddress)
{
    EnableSwitch1ChipSelect();
    
    uint24_t totalImageSize = GetSwitch1TotalSize();
    uint24_t flashAddress = (imageAddress * totalImageSize) + GetSwitch1ImageStartAddress();
    
    SendImageToSwitches(flashAddress, totalImageSize);
    
    DisableSwitch1ChipSelect();
    
    SendSwitch1ImageChangedNotification(imageAddress);
}

// Loads the image at the selected address (0001, 0002, etc.) on switch 2
void LoadSwitch2Image(uint24_t imageAddress)
{
    EnableSwitch2ChipSelect();
    
    uint24_t totalImageSize = GetSwitch2TotalSize();
    uint24_t flashAddress = (imageAddress * totalImageSize) + GetSwitch2ImageStartAddress();
    
    SendImageToSwitches(flashAddress, totalImageSize);
    
    DisableSwitch2ChipSelect();
    
    SendSwitch2ImageChangedNotification(imageAddress);
}

// Loads the image at the selected address (0001,0002, etc.) on the rocker switch
void LoadRockerImage(uint24_t imageAddress)
{
    EnableRockerChipSelect();
    
    uint24_t totalImageSize = GetSwitch1TotalSize();
    uint24_t flashAddress = (imageAddress * totalImageSize) + GetSwitch1ImageStartAddress();
    
    SendImageToRocker(flashAddress, totalImageSize);
    
    DisableRockerChipSelect();    
}

// Sends the data from the flash memory to the currently selected switch
void SendImageToSwitches(uint24_t flashAddress, unsigned int imageSize)
{    
    DisableSwitchCommandSending(); // When commands are disabled data is able to be sent    
    
    StartReadFlashMemory(flashAddress);
    for (unsigned int i = 0; i < imageSize; i += 1) // We send 8 bytes at a time
    {
        Spi2ImmediateWriteByte(ContinueReadFlashMemory());
    }
    
    FinishReadFlashMemory();
}

// Sends the data from the flash memory to the rocker switch
void SendImageToRocker(uint24_t flashAddress, unsigned int imageSize)
{
    unsigned char databuffer[8];            
    
    EnableRockerChipSelect();
    EnableSwitchCommandSending();    
    
    SPI2_Exchange8bit(ROCKER_WRITE_IMAGE);
    DisableSwitchCommandSending();
    
    for (unsigned int i = 0; i < imageSize; i += 8) // We send 8 bytes at a time
    {
        uint24_t address = flashAddress + i;
        ReadFlashMemory(address, databuffer, 8);        
        SPI2_Exchange8bitBuffer(databuffer, 8, NULL);                 
    }
    
    DisableRockerChipSelect();
}

// </editor-fold>

// <editor-fold defaultstate="collapsed" desc="Switch Press Monitor and Behavior">

// Scans both switches for being pressed, and does whatever behavior the attribute
// specifies if the switch was pressed
void ScanSwitchesForPress(void)
{
    if (CheckIfTimerExpired(TIMER_SWITCH_SCAN) == false)
    { 
       return; // Don't do anything if it's not time to check
    }
    
    if (CheckIfSwitch1Pressed() == true)
    {
        if (switch1PressedPreviously != true)
        {
            switch1PressedPreviously = true;
            DoSwitch1PressedBehavior();
            responsesEnum response = RESPONSE_SWITCH1_PRESSED;
            SendResponseOverSerial(response);
        }
    }
    else
    {
        if (switch1PressedPreviously == true)
        {      
            responsesEnum response = RESPONSE_SWITCH1_RELEASED;
            SendResponseOverSerial(response);
        }
        switch1PressedPreviously = false;
    }
    
    if (CheckIfSwitch2Pressed() == true)
    {
        if (switch2PressedPreviously != true)
        {
            switch2PressedPreviously = true;
            DoSwitch2PressedBehavior();
            responsesEnum response = RESPONSE_SWITCH2_PRESSED;
            SendResponseOverSerial(response);
        }
    }
    else
    {
        if (switch2PressedPreviously == true)
        {      
            responsesEnum response = RESPONSE_SWITCH2_RELEASED;
            SendResponseOverSerial(response);
        }
        switch2PressedPreviously = false;
    }    
    
    rockerPressEnum rockerPress = CheckIfRockerPressed();
    
    if (rockerPress != ROCKER_NONE_PRESSED)
    {
        if (rockerPressedPreviously != true)
        {
            rockerPressedPreviously = true;
            rockerPreviouslyPressedSwitch = rockerPress;
            DoRockerPressedBehavior(rockerPress);
            SendRockerPressedResponse(rockerPress);
        }
    }
    else
    {
        if (rockerPressedPreviously == true)
        {
            rockerPressedPreviously = false;
            SendRockerReleasedResponse(rockerPreviouslyPressedSwitch);
            rockerPreviouslyPressedSwitch = ROCKER_NONE_PRESSED;
        }
    }
    
    RestartTimer(TIMER_SWITCH_SCAN);
}

// Does the specific behavior for switch 1 being pressed
void DoSwitch1PressedBehavior(void)
{
    if (switch1AttributeTableInvalid == true) // Error catching
    {
        return; // Don't do anything if no attribute table loaded
    }
    
    uint24_t switch1flashAddress;
    uint24_t switch2flashAddress;
    
    // When switch 1 is pressed, change both switches to the addresses in the
    // switch1PressAddress and switch2PressAddress for switch 1, respectively
    uint24_t attributeSize = GetCurrentSwitchAttributeSize();
    
    if (switch1ImageAttributes.switch1PressAddress == 0x00) // Don't do anything if the press address is 0
    {
        switch1flashAddress = (switch1ImageAttributes.currentAddress * attributeSize) + GetSwitch1AttributeStartAddress();
    }
    else
    {
        switch1flashAddress = (switch1ImageAttributes.switch1PressAddress * attributeSize) + GetSwitch1AttributeStartAddress();   
    }    
    
    if (switch1ImageAttributes.switch2PressAddress == 0x00) // Don't do anything if the press address is 0
    {
        switch2flashAddress = (switch2ImageAttributes.currentAddress * attributeSize) + GetSwitch2AttributeStartAddress();
    }
    else
    {
        switch2flashAddress = (switch1ImageAttributes.switch2PressAddress * attributeSize) + GetSwitch2AttributeStartAddress();   
    }    
    
    ReadImageAttribute(switch1flashAddress, &switch1ImageAttributes);
    ReadImageAttribute(switch2flashAddress, &switch2ImageAttributes);
    
    LoadSwitch1Image(switch1ImageAttributes.currentAddress); 
    LoadSwitch2Image(switch2ImageAttributes.currentAddress);
    
    CheckIfSwitch1LoopTimerNeedsStarting();
    CheckIfSwitch2LoopTimerNeedsStarting();
}

// Does the specific behavior for switch 2 being pressed
void DoSwitch2PressedBehavior(void)
{
    if (switch2AttributeTableInvalid == true) // Error catching
    {
        return; // Don't do anything if no attribute table loaded
    }
    
    uint24_t switch1flashAddress;
    uint24_t switch2flashAddress;
    
    // When switch 2 is pressed, change both switches to the addresses in the
    // switch1PressAddress and switch2PressAddress for switch 2, respectively
    uint24_t attributeSize = GetCurrentSwitchAttributeSize();
    
    if (switch2ImageAttributes.switch2PressAddress == 0x00) // Don't do anything if the press address is 0
    {
        switch2flashAddress = (switch2ImageAttributes.currentAddress * attributeSize) + GetSwitch2AttributeStartAddress();
    }
    else
    {
        switch2flashAddress = (switch2ImageAttributes.switch2PressAddress * attributeSize) + GetSwitch2AttributeStartAddress();
    }    
    
    if (switch2ImageAttributes.switch1PressAddress == 0x00) // Don't do anything if the press address is 0
    {
        switch1flashAddress = (switch1ImageAttributes.currentAddress * attributeSize) + GetSwitch1AttributeStartAddress();
    }
    else
    {
        switch1flashAddress = (switch2ImageAttributes.switch1PressAddress * attributeSize) + GetSwitch1AttributeStartAddress();
    }    
    
    ReadImageAttribute(switch1flashAddress, &switch1ImageAttributes);
    ReadImageAttribute(switch2flashAddress, &switch2ImageAttributes);    
    
    LoadSwitch1Image(switch1ImageAttributes.currentAddress);
    LoadSwitch2Image(switch2ImageAttributes.currentAddress);    
    
    CheckIfSwitch1LoopTimerNeedsStarting();    
    CheckIfSwitch2LoopTimerNeedsStarting();
}

// Do the specific behavior for the rocker switch being pressed
void DoRockerPressedBehavior(rockerPressEnum rockerPress)
{
    uint24_t attributeSize = GetCurrentSwitchAttributeSize();
    uint24_t rockerAddress;
    
    if (rockerAttributeTableInvalid == true) // Error catching
    {
        return;
    }
    
    switch (rockerPress)
    {
        default:
        case ROCKER_MIDDLE_PRESSED:
            if (rockerImageAttributes.rockerMiddlePressAddress == 0x00) // Don't do anything if the press address is 0
            {
                rockerAddress = (rockerImageAttributes.currentAddress * attributeSize) + GetRockerAttributeStartAddress();
            }
            else
            {
                rockerAddress = (rockerImageAttributes.rockerMiddlePressAddress * attributeSize) + GetRockerAttributeStartAddress();
            }            
            break;
        case ROCKER_TOP_PRESSED:
            if (rockerImageAttributes.rockerTopPressAddress == 0x00)
            {
                rockerAddress = (rockerImageAttributes.currentAddress * attributeSize) + GetRockerAttributeStartAddress();
            }
            else
            {
                rockerAddress = (rockerImageAttributes.rockerTopPressAddress * attributeSize) + GetRockerAttributeStartAddress();   
            }            
            break;
        case ROCKER_BOTTOM_PRESSED:
            if (rockerImageAttributes.rockerBottomPressAddress == 0x00)
            {
                rockerAddress = (rockerImageAttributes.currentAddress * attributeSize) + GetRockerAttributeStartAddress();
            }
            else
            {
                rockerAddress = (rockerImageAttributes.rockerBottomPressAddress * attributeSize) + GetRockerAttributeStartAddress();   
            }            
            break;
    }
    
    ReadImageAttribute(rockerAddress, &rockerImageAttributes);
    LoadRockerImage(rockerImageAttributes.currentAddress);
    CheckIfRockerLoopTimerNeedsStarting();
}

// Sends the rocker switch pressed response to the host
void SendRockerPressedResponse(rockerPressEnum rockerPress)
{
    responsesEnum response;
    
    switch (rockerPress)
    {
        default:
        case ROCKER_MIDDLE_PRESSED:            
            response = RESPONSE_ROCKER_MIDDLE_PRESSED;
            break;
        case ROCKER_TOP_PRESSED:
            response = RESPONSE_ROCKER_TOP_PRESSED;
            break;
        case ROCKER_BOTTOM_PRESSED:
            response = RESPONSE_ROCKER_BOTTOM_PRESSED;
            break;
    }    
    
    SendResponseOverSerial(response);            
}

// Sends the rocker switch released response to the host
void SendRockerReleasedResponse(rockerPressEnum rockerPress)
{
    responsesEnum response;
    
    switch (rockerPress)
    {
        default:
        case ROCKER_MIDDLE_PRESSED:            
            response = RESPONSE_ROCKER_MIDDLE_RELEASED;
            break;
        case ROCKER_TOP_PRESSED:
            response = RESPONSE_ROCKER_TOP_RELEASED;
            break;
        case ROCKER_BOTTOM_PRESSED:
            response = RESPONSE_ROCKER_BOTTOM_RELEASED;
            break;
    }    
    
    SendResponseOverSerial(response);
}

// Based on attributes, sets timers for switch behaviors (changing images, changing
// backlight color, Loading new attributes, etc.). If one switch is pressed/timer
// expired, it can affect both switches depending on the attribute table values
void CheckSwitchBehavior(void)
{
    uint24_t attributeSize;
    uint24_t switch1flashAddress;      
    uint24_t switch2flashAddress;      
    uint24_t rockerFlashAddress;
        
    if (CheckIfTimerExpired(TIMER_SWITCH1_LOOP) == true) // This defaults to false if the timer isn't running
    {
        if (switch1ImageAttributes.loopCurrentAddress == switch1ImageAttributes.loopEndAddress) // If we are at the end of the loop
        {
            StopTimer(TIMER_SWITCH1_LOOP);
            
            attributeSize = GetCurrentSwitchAttributeSize();            
            
            // We check the other switch first because we may be about to change what we're reading
            if (switch1ImageAttributes.switch2TimerExpiredJumpAddress == 0x0) // A zero means stay at the same address
            {
                // Don't do anything for switch 2, let it keep doing its thing
            }
            else
            {
                switch2flashAddress = (switch1ImageAttributes.switch2TimerExpiredJumpAddress * attributeSize) + GetSwitch2AttributeStartAddress();
                ReadImageAttribute(switch2flashAddress, &switch2ImageAttributes);
                LoadSwitch2Image(switch2ImageAttributes.currentAddress);
                CheckIfSwitch2LoopTimerNeedsStarting();
                switch2ImageAttributes.loopCurrentAddress = switch2ImageAttributes.currentAddress; // Need to re-set the loop address
            }
            
            if (switch1ImageAttributes.switch1TimerExpiredJumpAddress == 0x0) // A zero means stay at the same address
            {
                switch1ImageAttributes.loopCurrentAddress = switch1ImageAttributes.currentAddress;
                LoadSwitch1Image(switch1ImageAttributes.loopCurrentAddress);
                CheckIfSwitch1LoopTimerNeedsStarting();
            }
            else
            {
                switch1flashAddress = (switch1ImageAttributes.switch1TimerExpiredJumpAddress * attributeSize) + GetSwitch1AttributeStartAddress();
                ReadImageAttribute(switch1flashAddress, &switch1ImageAttributes);
                LoadSwitch1Image(switch1ImageAttributes.currentAddress);
                CheckIfSwitch1LoopTimerNeedsStarting();
                switch1ImageAttributes.loopCurrentAddress = switch1ImageAttributes.currentAddress; // Need to re-set the loop address
            }        
        }
        else // If we're not at the end address increment the counter, load the new image, and re-start the timer
        {
            switch1ImageAttributes.loopCurrentAddress++;
            LoadSwitch1Image(switch1ImageAttributes.loopCurrentAddress);
            RestartTimer(TIMER_SWITCH1_LOOP);
        }
        
        responsesEnum response = RESPONSE_SWITCH1_TIMER_EXPIRED;
        SendResponseOverSerial(response);
    }
       
    if (CheckIfTimerExpired(TIMER_SWITCH2_LOOP) == true) // This defaults to false if the timer isn't running
    {
        if (switch2ImageAttributes.loopCurrentAddress == switch2ImageAttributes.loopEndAddress)
        {
            StopTimer(TIMER_SWITCH2_LOOP);
            
            attributeSize = GetCurrentSwitchAttributeSize();
            
            // We check the other switch first because we may be about to change what we're reading
            if (switch2ImageAttributes.switch1TimerExpiredJumpAddress == 0x0) // A zero means stay at the same address
            {
                // Don't do anything for switch 1, let it keep doing its thing
            }
            else
            {
                switch1flashAddress = (switch2ImageAttributes.switch1TimerExpiredJumpAddress * attributeSize) + GetSwitch1AttributeStartAddress();
                ReadImageAttribute(switch1flashAddress, &switch1ImageAttributes);
                LoadSwitch1Image(switch1ImageAttributes.currentAddress);
                CheckIfSwitch1LoopTimerNeedsStarting();
                switch1ImageAttributes.loopCurrentAddress = switch1ImageAttributes.currentAddress; // Need to re-set the loop address
            }
            
            if (switch2ImageAttributes.switch2TimerExpiredJumpAddress == 0x0) // A zero means stay at the same address
            {
                switch2ImageAttributes.loopCurrentAddress = switch2ImageAttributes.currentAddress;
                LoadSwitch2Image(switch2ImageAttributes.loopCurrentAddress);
                CheckIfSwitch2LoopTimerNeedsStarting();
            }
            else
            {
                switch2flashAddress = (switch2ImageAttributes.switch2TimerExpiredJumpAddress * attributeSize) + GetSwitch2AttributeStartAddress();
                ReadImageAttribute(switch2flashAddress, &switch2ImageAttributes);
                LoadSwitch2Image(switch2ImageAttributes.currentAddress);
                CheckIfSwitch2LoopTimerNeedsStarting();
                switch2ImageAttributes.loopCurrentAddress = switch2ImageAttributes.currentAddress; // Need to re-set the loop address
            }       
        }
        else // If we're not at the end address increment the counter, load the new image, and re-start the timer
        {
            
            switch2ImageAttributes.loopCurrentAddress++;
            LoadSwitch2Image(switch2ImageAttributes.loopCurrentAddress);
            RestartTimer(TIMER_SWITCH2_LOOP);
        }
        
        responsesEnum response = RESPONSE_SWITCH2_TIMER_EXPIRED;
        SendResponseOverSerial(response);        
    }        
    
    if (CheckIfTimerExpired(TIMER_ROCKER_LOOP) == true) // This defaults to false if the timer isn't running
    {
        if (rockerImageAttributes.loopCurrentAddress == rockerImageAttributes.loopEndAddress)
        {
            StopTimer(TIMER_ROCKER_LOOP);
            
            attributeSize = GetCurrentSwitchAttributeSize();                 
            
            if (rockerImageAttributes.switch1TimerExpiredJumpAddress == 0x0) // A zero means stay at the same address
            {
                rockerImageAttributes.loopCurrentAddress = rockerImageAttributes.currentAddress;
                LoadRockerImage(rockerImageAttributes.loopCurrentAddress);
                CheckIfRockerLoopTimerNeedsStarting();
            }
            else
            {
                rockerFlashAddress = (rockerImageAttributes.switch1TimerExpiredJumpAddress * attributeSize) + GetSwitch2AttributeStartAddress();
                ReadImageAttribute(rockerFlashAddress, &rockerImageAttributes);
                LoadRockerImage(rockerImageAttributes.currentAddress);
                CheckIfRockerLoopTimerNeedsStarting();
                rockerImageAttributes.loopCurrentAddress = rockerImageAttributes.currentAddress; // Need to re-set the loop address
            }       
        }
        else // If we're not at the end address increment the counter, load the new image, and re-start the timer
        {
            rockerImageAttributes.loopCurrentAddress++;
            LoadRockerImage(rockerImageAttributes.loopCurrentAddress);
            RestartTimer(TIMER_ROCKER_LOOP);
        }
        
        responsesEnum response = RESPONSE_SWITCH1_TIMER_EXPIRED;
        SendResponseOverSerial(response);
    }
}

// If the timer1 attribute isn't 0, there is a loop and need to start a timer to cycle through images
void CheckIfSwitch1LoopTimerNeedsStarting(void)
{
    if (switch1AttributeTableInvalid == true) // Error catching
    {
        return; // Don't do anything if no attribute table loaded
    }
     
    if (switch1ImageAttributes.timer1 != 0)
    {
        unsigned int timeToShowImagesInMs;
        if (switch1ImageAttributes.timer2 == 0)
        {
            timeToShowImagesInMs = (unsigned int) (switch1ImageAttributes.timer1 * 0x100); // A 0 in timer 2 equals 256
        }    
        else
        {
            timeToShowImagesInMs = (unsigned int) (switch1ImageAttributes.timer1 * switch1ImageAttributes.timer2); // Multiply the timers together for the duration
        }
        StartTimer(TIMER_SWITCH1_LOOP, timeToShowImagesInMs);
        switch1ImageAttributes.loopCurrentAddress = switch1ImageAttributes.currentAddress; // Need to re-set the loop address
    }
    else
    {
        StopTimer(TIMER_SWITCH1_LOOP);
    }
}

// If the timer1 attribute isn't 0, there is a loop and need to start a timer to cycle through images
void CheckIfSwitch2LoopTimerNeedsStarting(void)
{
    if (switch2AttributeTableInvalid == true) // Error catching
    {
        return; // Don't do anything if no attribute table loaded
    }
     
    if (switch2ImageAttributes.timer1 != 0)
    {
        unsigned int timeToShowImagesInMs;
        if (switch2ImageAttributes.timer2 == 0)
        {
            timeToShowImagesInMs = (unsigned int) (switch2ImageAttributes.timer1 * 0x100); // A 0 in timer 2 equals 256
        }    
        else
        {
            timeToShowImagesInMs = (unsigned int) (switch2ImageAttributes.timer1 * switch2ImageAttributes.timer2); // Multiply the timers together for the duration
        }
        StartTimer(TIMER_SWITCH2_LOOP, timeToShowImagesInMs);
        switch2ImageAttributes.loopCurrentAddress = switch2ImageAttributes.currentAddress; // Need to re-set the loop address
    }
    else
    {
        StopTimer(TIMER_SWITCH2_LOOP);
    }
}

// If the timer1 attribute isn't 0, there is a loop and need to start a timer to cycle through images
void CheckIfRockerLoopTimerNeedsStarting(void)
{
    if (rockerAttributeTableInvalid == true) // Error catching
    {
        return; // Don't do anything if no attribute table loaded
    }
     
    if (rockerImageAttributes.timer1 != 0)
    {
        unsigned int timeToShowImagesInMs;
        if (rockerImageAttributes.timer2 == 0)
        {
            timeToShowImagesInMs = (unsigned int) (rockerImageAttributes.timer1 * 0x100); // A 0 in timer 2 equals 256
        }    
        else
        {
            timeToShowImagesInMs = (unsigned int) (rockerImageAttributes.timer1 * rockerImageAttributes.timer2); // Multiply the timers together for the duration
        }
        StartTimer(TIMER_ROCKER_LOOP, timeToShowImagesInMs);
        rockerImageAttributes.loopCurrentAddress = rockerImageAttributes.currentAddress; // Need to re-set the loop address
    }
    else
    {
        StopTimer(TIMER_ROCKER_LOOP);
    }
}

// Manually changes the images to the ones specified in the parameters
void SetSwitchImages(uint24_t switch1Address, uint24_t switch2Address)
{
    uint24_t flashAddress;
    uint24_t attributeSize;
    uint24_t switch1AttributeStartAddress;
    uint24_t switch2AttributeStartAddress;
    
    switch1AttributeStartAddress = GetSwitch1AttributeStartAddress();
    switch2AttributeStartAddress = GetSwitch2AttributeStartAddress();
    
    attributeSize = GetCurrentSwitchAttributeSize();
    
    if (GetCurrentDevKit() == DEV_KIT_ROCKER)
    {
        if (rockerAttributeTableInvalid == false) // Load the attribute if it is valid, otherwise just load the image
        {            
            flashAddress = (uint24_t) ((switch1Address * attributeSize) + switch1AttributeStartAddress); // We define the flash attributes to be in a specific part of memory    
            ReadImageAttribute(flashAddress, &rockerImageAttributes); // Loads the attributes into the struct
        }
        else
        {
            rockerImageAttributes.currentAddress = switch1Address;
        }
        
        LoadRockerImage(rockerImageAttributes.currentAddress);  // Loads the image on to the switch
    }
    else
    {
        if (switch1AttributeTableInvalid == false) // Load the attribute if it is valid, otherwise just load the image
        {
            flashAddress = (uint24_t) ((switch1Address * attributeSize) + switch1AttributeStartAddress); // We define the flash attributes to be in a specific part of memory
            ReadImageAttribute(flashAddress, &switch1ImageAttributes); // Loads the attributes into the struct
        }   
        else
        {
            switch1ImageAttributes.currentAddress = switch1Address;
        }
        
        if (switch2AttributeTableInvalid == false) // Load the attribute if it is valid, otherwise just load the image
        {
            flashAddress = (uint24_t) ((switch2Address * attributeSize) + switch2AttributeStartAddress);
            ReadImageAttribute(flashAddress, &switch2ImageAttributes); // Loads the attributes into the struct   
        }
        else
        {
            switch2ImageAttributes.currentAddress = switch2Address;
        }
        
        LoadSwitch1Image(switch1ImageAttributes.currentAddress);  // Loads the image on to the switch
        LoadSwitch2Image(switch2ImageAttributes.currentAddress);
    }                     
    
    CheckIfSwitch1LoopTimerNeedsStarting();
    CheckIfSwitch2LoopTimerNeedsStarting();
    CheckIfRockerLoopTimerNeedsStarting();
}

// Sends a response to the host that the switch 1 image changed
void SendSwitch1ImageChangedNotification(uint24_t imageAddress)
{
    char addressHigh    = (imageAddress & 0xFF00) >> 8;
    char addressLow     = (imageAddress & 0x00FF);
    char convertedAsciiBuffer[2];
    
    SendResponseOverSerial(RESPONSE_SWITCH1_CHANGED_IMAGE);    
    
    ConvertAsciiToAsciiHex(addressHigh,convertedAsciiBuffer);
    SendResponseOverSerial(convertedAsciiBuffer[0]);
    SendResponseOverSerial(convertedAsciiBuffer[1]);
    
    ConvertAsciiToAsciiHex(addressLow,convertedAsciiBuffer);
    SendResponseOverSerial(convertedAsciiBuffer[0]);
    SendResponseOverSerial(convertedAsciiBuffer[1]);    
}

// Sends a response to the host that the switch 2 image changed
void SendSwitch2ImageChangedNotification(uint24_t imageAddress)
{
    char addressHigh    = (imageAddress & 0xFF00) >> 8;
    char addressLow     = (imageAddress & 0x00FF);
    char convertedAsciiBuffer[2];
    
    SendResponseOverSerial(RESPONSE_SWITCH2_CHANGED_IMAGE);    
    
    ConvertAsciiToAsciiHex(addressHigh,convertedAsciiBuffer);
    SendResponseOverSerial(convertedAsciiBuffer[0]);
    SendResponseOverSerial(convertedAsciiBuffer[1]);
    
    ConvertAsciiToAsciiHex(addressLow,convertedAsciiBuffer);
    SendResponseOverSerial(convertedAsciiBuffer[0]);
    SendResponseOverSerial(convertedAsciiBuffer[1]);    
}

// </editor-fold>

// <editor-fold defaultstate="collapsed" desc="Character Display Functions">

unsigned char _lineColor[2];
unsigned char _characterBackgroundForegroundColor[4];

// Buffer with 2 bytes of line color
void SetCharacterLineColor(unsigned char * lineColor)
{
    // 2 bytes per color in BGR 565 format
    _lineColor[0] = lineColor[0];
    _lineColor[1] = lineColor[1];
}

// Buffer with 4 bytes, 2 bytes of background color followed by 2 bytes of foreground color
void SetCharacterBackGroundForegroundColor(unsigned char * colorBuffer)
{
    // 2 bytes of background color
    _characterBackgroundForegroundColor[0] = colorBuffer[0];
    _characterBackgroundForegroundColor[1] = colorBuffer[1];
    // 2 bytes of foreground color
    _characterBackgroundForegroundColor[2] = colorBuffer[2];
    _characterBackgroundForegroundColor[3] = colorBuffer[3];
}

// Manually sets brightness for both switches
void SetSwitchBrightness(unsigned char brightness)
{
    unsigned char parameters[1] = {0};
    unsigned char oledBrightness = (unsigned char)(brightness & 0xF); // Per datasheets the brightness has 16 steps, from 0-F
    
    parameters[0] = oledBrightness;
    
    // The OLED switches and rocker have different brightness commands
    if (GetCurrentDevKit() == DEV_KIT_ROCKER)
    {        
        SendRockerConfigurationCommand(ROCKER_BRIGHTNESS, parameters, 1);
    }
    else
    {
        EnableSwitch1ChipSelect();
        EnableSwitch2ChipSelect();
    
        SendSwitchConfigurationCommand(SWITCH_DISPLAY_OFF, parameters, 0);        
        SendSwitchConfigurationCommand(SWITCH_MASTER_CURRENT_CONTROL, parameters, 1);        
        SendSwitchConfigurationCommand(SWITCH_DISPLAY_ON, parameters, 0);
        
        DisableSwitch1ChipSelect();
        DisableSwitch2ChipSelect(); 
    }            
}

void SendCharacterToSwitch(unsigned char * colorData)
{        
    DisableSwitchCommandSending(); // When commands are disabled data is able to be sent        
    SPI2_Exchange8bitBuffer(colorData, 2, NULL);                      
}

void AddCharacterToDisplay(unsigned char switchNumber,
                            unsigned char row, unsigned char column,
                            unsigned char numberOfCharacters, unsigned char * characters)
{
    // CURRENTLY SET UP FOR FRAMELESS ONLY    
    unsigned char parameters[2];
    const uint8_t * fontData;
    int i = 0; // Counter variable
    int j = 0; // Counter variable
    int k = 0; // Counter variable
    unsigned int bitMask = 0;
    
    /* If the user set the colors the same or just started up, set default of black text on white background*/
    if ((_characterBackgroundForegroundColor[0] == _characterBackgroundForegroundColor[2]) && (_characterBackgroundForegroundColor[1] == _characterBackgroundForegroundColor[3]))
    {
        _characterBackgroundForegroundColor[0] = 0xFF;
        _characterBackgroundForegroundColor[1] = 0xFF;
        _characterBackgroundForegroundColor[2] = 0x00;
        _characterBackgroundForegroundColor[3] = 0x00;
    }
           
    if (switchNumber == 0x00)
    {
        EnableSwitch1ChipSelect();
    }        
    else
    {
        EnableSwitch2ChipSelect();
    }                                        
    
    for (i = 0; i < numberOfCharacters; i++)
    {
        fontData = GetFontCharacter(characters[i]);
        
        parameters[0] = (unsigned char)(column + i * 8); // We multiply by 7+1 to give a pixel's worth of space between characters
        parameters[1] = (unsigned char)(parameters[0] + 7); // The font size is 7x10, so the window is the start plus 7
        SendSwitchConfigurationCommand(SWITCH_WINDOW_COLUMN, parameters, 2);        
    
        parameters[0] = row;
        parameters[1] = (unsigned char)(parameters[0] + 10); // The font size is 7x10, so the window is the start plus 10
        SendSwitchConfigurationCommand(SWITCH_WINDOW_ROW, parameters, 2);  
        
        for (j = 0; j < 10; j++)
        {                  
            for (k = 7; k >= 0; k--)
            {
                bitMask = (unsigned int)(1 << k);
                if ((bitMask & (fontData[j])) > 0) // If the bit we're one has font data
                {
                    SendCharacterToSwitch(&(_characterBackgroundForegroundColor[2])); // Send foreground data
                }
                else
                {
                    SendCharacterToSwitch(&(_characterBackgroundForegroundColor[0])); // Send background data
                }
            }         
        }        
    }
    
    if (switchNumber == 0x00)
    {
        DisableSwitch1ChipSelect();    
    }    
    else
    {
        DisableSwitch2ChipSelect();    
    }
        
    /* Reset everything after we finish in case they send an image */
    parameters[0] = FRAMELESS_WINDOW_COLUMN_DEFAULT_1;
    parameters[1] = FRAMELESS_WINDOW_COLUMN_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_COLUMN, parameters, 2);
   
    parameters[0] = FRAMELESS_WINDOW_ROW_DEFAULT_1;
    parameters[1] = FRAMELESS_WINDOW_ROW_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_ROW, parameters, 2);    
}

void AddLineToSwitch(unsigned char switchNumber, unsigned char numberOfRows, unsigned char startingRow)
{
    unsigned char parameters[2];
    unsigned int numberOfBytesToWrite = 0;
    unsigned int i = 0;
    
    if (switchNumber == 0x00)
    {
        EnableSwitch1ChipSelect();
    }
    else
    {
        EnableSwitch2ChipSelect();
    }
    
    parameters[0] = 0; // Start at the first column
    parameters[1] = 95; // End at the last column
    SendSwitchConfigurationCommand(SWITCH_WINDOW_COLUMN, parameters, 2);        
    
    parameters[0] = startingRow;
    parameters[1] = (unsigned char)(startingRow + numberOfRows - 1); // Fill in a certain number of rows
    SendSwitchConfigurationCommand(SWITCH_WINDOW_ROW, parameters, 2);  
    
    numberOfBytesToWrite = (unsigned char)(numberOfRows * 96); // 96 columns per row
    
    for (i = 0; i < numberOfBytesToWrite; i++)
    {        
        SPI2_Exchange8bitBuffer(_lineColor, 2, NULL);
    }        
    
     /* Reset everything after we finish in case they send an image */
    parameters[0] = FRAMELESS_WINDOW_COLUMN_DEFAULT_1;
    parameters[1] = FRAMELESS_WINDOW_COLUMN_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_COLUMN, parameters, 2);
   
    parameters[0] = FRAMELESS_WINDOW_ROW_DEFAULT_1;
    parameters[1] = FRAMELESS_WINDOW_ROW_DEFAULT_2;
    SendSwitchConfigurationCommand(SWITCH_WINDOW_ROW, parameters, 2);    
    
    if (switchNumber == 0x00)
    {
        DisableSwitch1ChipSelect();    
    }    
    else
    {
        DisableSwitch2ChipSelect();    
    }
}

// </editor-fold>
