/*
 * NKK Switches S02L1 OLED Main.c file
 * v1.8.82
 * This software has a few assumptions. First, all non-commands should be sent
 * using ASCII hex. IE if the user wants to load image 001, they would send the 
 * load image command (28) followed by: 0x30 0x30 0x30 0x31 (three ASCII zeroes and
 * and ASCII 1). Second, all addressing should be sent in 2 bytes. IE address one
 * is actually 00 01 (then converted to ASCII hex).
 * 
 * THE DEV KIT MUST BE CHANGED IN GetCurrentDevKit TO WHAT DEV KIT YOU ARE USING. 
 * If you do not, the program may not function
 * correctly until the host sends the COMMAND_VERSATILE command followed by the 
 * VESTATILE_SUBCOMMAND1_CHANGE_DEVKIT followed by the devKitTypeEnum number
 * of the dev kit being used (in ASCII hex)
 * 
 * The Microchip Code Configurator (MCC) was used to easily generate the pin outs,
 * and we just interface to their generated files. A few of the modules were changed
 * slightly to accommodate our code. Timer 0 was set up for 1 ms interrupt intervals.
 */

#include "main.h"
#include "mcc_generated_files/mcc.h"
#include "Communication_Handler.h"
#include "Setup_Program.h"
#include "Flash_Functions.h"
#include "Switch_Functions.h"
#include "EEPROM_Functions.h"
#include "IO_Functions.h"

devKitTypeEnum currentDevKit = DEV_KIT_UNASSIGNED;

// Gets the version of software in ASCII hex
void GetSoftwareVersion(unsigned int* version)
{
    version[0] = 0x31; // Major version 1
    version[1] = 0x38; // Minor version 5
}

/*
                         Main application
 */
void main(void)
{
    // Initialize the device
    SYSTEM_Initialize();

    // Enable high priority global interrupts
    INTERRUPT_GlobalInterruptHighEnable();

    // Enable low priority global interrupts.
    INTERRUPT_GlobalInterruptLowEnable();

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();

    __delay_ms(50); // We put in a delay to allow all peripherals (switches, memory, etc.) to boot up
    SetupProgram();        
    
    while (1)
    {                           
        HandleSerialCommunications();        
        ScanSwitchesForPress();
        CheckSwitchBehavior();        
    }
}   
    
    // Gets the current development kit this program is coded for. This is loaded
    // in the setup attribute only. It defaults to 2 frameless
    devKitTypeEnum GetCurrentDevKit(void)
    {
        if (currentDevKit == DEV_KIT_UNASSIGNED)
        {
            currentDevKit = ReadDevKitFromEeprom();
        }
        
        // If it is still unassigned, give it a default
        if (currentDevKit == DEV_KIT_UNASSIGNED)
        {
            currentDevKit = DEV_KIT_TWO_FRAMELESS; // CHANGE THIS TO WHAT DEV KIT YOU ARE USING
            WriteDevKitToEeprom(currentDevKit);
        }
                
        return currentDevKit;
    }
    
    // Sets the current development kit this program is coded for
    void SetCurrentDevKit(devKitTypeEnum kit)
    {
        currentDevKit = kit;
    }
    
    // Puts the controller name in ASCII hex in the provided buffer
    void GetControllerName(char* controllerName)
    {
        // The name of this demonstration board is S02G1
        controllerName[0] = 0x53; // S in ASCII hex
        controllerName[1] = 0x30; // 0 in ASCII hex
        controllerName[2] = 0x32; // 2 in ASCII hex
        controllerName[3] = 0x4C; // L in ASCII hex
        controllerName[4] = 0x31; // 1 in ASCII hex
    }
    
    // Gets the current switch total byte size. This is needed for parsing serial
    // commands from the host. If coding for a different switch, change
    // the code here
    unsigned int GetSwitchTotalSize(unsigned char switchCode)
    {
        switch(switchCode)    
        {
            default:
            case SWITCH_TYPE_96x64_OLED: // 96x64(x2 bytes/colors) = 12288 bytes, x 512 images = 0x600000 bytes
                return 12288;
            case SWITCH_TYPE_64x48_OLED: // 64x48 (x2 bytes/colors) = 6144 bytes, x 512 images = 0x300000 bytes
                return 6144;
            case SWITCH_TYPE_52x36_OLED: // 52x36 (x2 bytes/colors) = 3744 (we byte align to 3840), x 512 images = 0x1E0000 bytes
                return 3744;
            case SWITCH_TYPE_OLED_ROCKER: // 96*64 bytes (byte aligned to 1024) x 512 images = 0x080000 bytes 
                return 768;
        }
    }
    
    // Gets the current switch attribute byte size. This is needed for parsing serial
    // commands from the host. If coding for a different switch, change
    // the code here
    unsigned char GetCurrentSwitchAttributeSize(void)
    {
        return 16; // The attribute block for all OLED is 16 bytes
    }
    
    unsigned int GetSwitch1TotalSize(void)
    {
        unsigned int size;
        devKitTypeEnum type = GetCurrentDevKit();
        
        switch(type)
        {
            default:
            case DEV_KIT_TWO_FRAMELESS: // Switch 1 in this kit is frameless
                size = GetSwitchTotalSize(SWITCH_TYPE_96x64_OLED);
                break;
            case DEV_KIT_ONE_FRAMELESS_ONE_STANDARD: // Switch 1 in this kit is frameless
                size = GetSwitchTotalSize(SWITCH_TYPE_96x64_OLED);
                break;
            case DEV_KIT_ONE_STANDARD_ONE_DISPLAY: // Switch 1 in this kit is a display
                size = GetSwitchTotalSize(SWITCH_TYPE_52x36_OLED);
                break;
            case DEV_KIT_ROCKER: // Switch 1 in this kit is a rocker
                size = GetSwitchTotalSize(SWITCH_TYPE_OLED_ROCKER);
                break;
        }
        
        return size;
    }
    
    unsigned int GetSwitch2TotalSize(void)
    {
        unsigned int size;
        devKitTypeEnum type = GetCurrentDevKit();
        
        switch(type)
        {
            default:
            case DEV_KIT_TWO_FRAMELESS: // Switch 2 in this kit is frameless
                size = GetSwitchTotalSize(SWITCH_TYPE_96x64_OLED);
                break;
            case DEV_KIT_ONE_FRAMELESS_ONE_STANDARD: // Switch 2 in this kit is standard
                size = GetSwitchTotalSize(SWITCH_TYPE_64x48_OLED);
                break;
            case DEV_KIT_ONE_STANDARD_ONE_DISPLAY: // Switch 2 in this kit is standard
                size = GetSwitchTotalSize(SWITCH_TYPE_64x48_OLED);
                break;
            case DEV_KIT_ROCKER: // There is no switch 2 in this kit
                size = GetSwitchTotalSize(SWITCH_TYPE_OLED_ROCKER);
                break;
        }
        
        return size;
    }
    
    // Checks if the supplied switch code is valid
    bool CheckIfSwitchCodeValid(unsigned char code)
    {
        if (code == SWITCH_TYPE_96x64_OLED)
        {
            return true;   
        }   
        else if (code == SWITCH_TYPE_64x48_OLED)
        {
            return true;            
        }
        else if (code == SWITCH_TYPE_52x36_OLED)
        {
            return true;            
        }
        else if (code == SWITCH_TYPE_OLED_ROCKER)
        {
            return true;
        }
        
        return false;
    }
    
    // The data from the host arrives in a hex value, so we need
    // to convert to an ASCII value for creating a nibble (half a byte)
    unsigned char ConvertAsciiHexToAscii(unsigned char data)
    {
        unsigned char result;
        
        if (data >= 0x30 && data <=  0x3F) // If the ASCII value is 0-9 (a number)
        {
            result = (unsigned char) (data - 0x30);
        }
        else if (data >= 0x41 && data <= 0x46) // Or if the ASCII value is A-F
        {
            result = (unsigned char) (data - 0x37);
        }
        else if (data >= 0x61 && data <= 0x66) // Or if the ASCII value is a-f
        {
            result = (unsigned char) (data - 0x57);
        }
        else
        {
            return RESPONSE_UNSUCCESSFUL;
        }
        
        return result;
    }
    
    // To send data out over serial, convert the ASCII value to two bytes
    // of hex values for ASCII to ensure no commands are being sent
    bool ConvertAsciiToAsciiHex(unsigned char data, unsigned char* convertedData)
    {
        unsigned char dataNibble[2];
        
        dataNibble[0] = (unsigned char) ((data & 0xF0) >> 4); // put each nibble in the buffer
        dataNibble[1] = (unsigned char) (data & 0x0F);
        
        for (unsigned char i = 0; i < 2; i++) // Convert both nibbles
        {
            if (dataNibble[i] < 0xA) // If the ASCII value is 0-9
            {
                convertedData[i] = (unsigned char) (dataNibble[i] + 0x30);
            }
            else if (dataNibble[i] >= 0xA) // Or if the ASCII value is A-F
            {
                convertedData[i] = (unsigned char) (dataNibble[i] + 0x37);
            }     
            else
            {
                return false; // Otherwise this fails
            }   
        }
        
        return true; // Return true if successful
    }      
/**
 End of File
*/